#include <assert.h>
#include <time.h>
#include <alsa/asoundlib.h>

int main(int argc, char *argv[]) {
    const char *dev;
    int cap, err;
    snd_pcm_hw_params_t *hwparams;
    snd_pcm_sw_params_t *swparams;
    snd_pcm_status_t *status;
    snd_pcm_t *pcm;
    snd_output_t *output = NULL;

    unsigned rate;
    unsigned periods = 2;
    unsigned int this_time, buffer_time;
    int sleep_time = 1000;
    int buffer[65536];
    int stat[65536];
    snd_pcm_uframes_t boundary, buffer_size, period_size, this_period;
    int dir = 1;
    int i, counter;
    snd_pcm_sframes_t avail;

    snd_pcm_hw_params_alloca(&hwparams);
    snd_pcm_sw_params_alloca(&swparams);

    dev = argc > 1 ? argv[1] : "hw:0,0";
    cap = argc > 2 ? atoi(argv[2]) : 0;
    rate = argc > 3 ? atoi(argv[3]) : 44100;

    err = snd_pcm_open(&pcm, dev, cap == 0 ? SND_PCM_STREAM_PLAYBACK : 
		SND_PCM_STREAM_CAPTURE, 0);
    if (err < 0) {
	printf("snd_pcm_open fail %s err %s\n",dev, snd_strerror(err));
        exit(0);
    }

    err = snd_pcm_hw_params_any(pcm, hwparams);
//  assert(err == 0);
/*
    err = snd_pcm_hw_params_set_rate_resample(pcm, hwparams, 0);
    assert(err == 0);
*/
    err = snd_pcm_hw_params_set_access(pcm, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED);
    assert(err == 0);

    err = snd_pcm_hw_params_set_format(pcm, hwparams, SND_PCM_FORMAT_S16_LE);
    assert(err == 0);

    err = snd_pcm_hw_params_set_rate_near(pcm, hwparams, &rate, NULL);
    assert(err == 0);

    err = snd_pcm_hw_params_set_channels(pcm, hwparams, 2);
    assert(err == 0);
/*
    err = snd_pcm_hw_params_set_periods_integer(pcm, hwparams);
    assert(err == 0);
*/

    dir = 0;
    err = snd_pcm_hw_params_set_periods_near(pcm, hwparams, &periods, &dir);
    assert(err == 0);

    buffer_size = rate * 2;
    err = snd_pcm_hw_params_set_buffer_size_near(pcm, hwparams, &buffer_size);
    assert(err == 0);

    err = snd_pcm_hw_params(pcm, hwparams);
    assert(err == 0);

    err = snd_pcm_hw_params_current(pcm, hwparams);
    assert(err == 0);

    err = snd_pcm_sw_params_current(pcm, swparams);
    assert(err == 0);

/*
    if (cap == 0)
      err = snd_pcm_sw_params_set_avail_min(pcm, swparams, 1);
    else
      err = snd_pcm_sw_params_set_avail_min(pcm, swparams, 0);
    assert(err == 0);
*/
/*
    err = snd_pcm_sw_params_set_period_event(pcm, swparams, 0);
    assert(err == 0);
*/
    err = snd_pcm_hw_params_get_buffer_size(hwparams, &buffer_size);
    assert(err == 0);
/*
    err = snd_pcm_hw_params_get_buffer_time(hwparams, &buffer_time, 0);
    assert(err == 0);
*/
    err = snd_pcm_hw_params_get_period_size(hwparams, &period_size, 0);
    assert(err == 0);
/*
    err = snd_pcm_sw_params_set_start_threshold(pcm, swparams, buffer_size);
    assert(err == 0);

    err = snd_pcm_sw_params_get_boundary(swparams, &boundary);
    assert(err == 0);

    err = snd_pcm_sw_params_set_stop_threshold(pcm, swparams, buffer_size);
    assert(err == 0);
*/
    err = snd_pcm_sw_params_set_tstamp_mode(pcm, swparams, SND_PCM_TSTAMP_ENABLE);
    assert(err == 0);

    err = snd_pcm_sw_params(pcm, swparams);
    assert(err == 0);


    err = snd_pcm_sw_params_current(pcm, swparams);
    assert(err == 0);

    err = snd_output_stdio_attach(&output, stdout, 0);
    if (err < 0) 
	printf("Output failed: %s\n", snd_strerror(err));
    snd_pcm_dump(pcm, output);

/*     assert(snd_pcm_hw_params_is_monotonic(hwparams) > 0); */


    if (cap) {
      err = snd_pcm_start(pcm);
      assert(err == 0);
    }
    else {
        this_period = 0;
        while (this_period<buffer_size) {
            err = snd_pcm_writei(pcm, buffer, period_size);
	    printf("pcm write %d\n",err);
            this_period += period_size;
	};
    };

    avail=0;
    this_time =0;
    counter = 0;
//  printf("buffer time %d\n",buffer_time);
    while (counter < 32767 && avail >= 0) {
        avail = snd_pcm_avail(pcm);
        stat[counter] = avail;
        usleep(sleep_time);
        counter++;
        this_time += sleep_time;
    };
    for (i=0; i<counter; i++) 
        printf("%6d %d\n",i, stat[i]);

    snd_pcm_close(pcm);
    return 0;
}